// #define DEBUG
#define Z80_MAGIC 0x6f30387a

struct z80fileinfo
{
    unsigned long header;
    int code_length;
    int relocentries;
    int externs;
    int globals;
};


struct relocentry
{
    unsigned short offset;	/* offset in file to relocate */
};

struct globalsym
{
    char name[20];		/* symbol name */
    unsigned short offset;	/* offset in code section */
};

#define RELOCMODE_ABSOLUTE 1
#define RELOCMODE_RELATIVE 2


struct externsym
{
    char name[20];		/* sybol name */
    unsigned char mode;
    unsigned short offset;	/* offset to insert the symbol at in code section */
};

unsigned char * load(char * filename, int * slen);
void make_global(unsigned char * name);
void add_sym_reference(unsigned char * name, unsigned short pc);
void add_relsym_reference(unsigned char * name, unsigned short pc);
void add_symbol(unsigned char * name, unsigned short pc);
void add_reloc_entry(unsigned short pc);
void complete_symbol_values(void);
void save(int, int);
void putbyte(unsigned char b);

void assemble_word(char*);
void assemble_byte(char*);
void assemble_dsbyte(char*);
void assemble_ld(char*);
void assemble_push(char *);
void assemble_pop(char *);
void assemble_ex(char *);
void assemble_exx(char *);
void assemble_add(char *);
void assemble_adc(char *);
void assemble_sbc(char *);
void assemble_sub(char *);
void assemble_cp(char *);
void assemble_and(char *);
void assemble_or(char *);
void assemble_xor(char *);
void assemble_inc(char *);
void assemble_dec(char *);
void assemble_cpl(char *);
void assemble_neg(char *);
void assemble_scf(char *);
void assemble_ccf(char *);
void assemble_daa(char *);
void assemble_rlca(char *);
void assemble_rrca(char *);
void assemble_rla(char *);
void assemble_rra(char *);
void assemble_rlc(char *);
void assemble_rrc(char *);
void assemble_rl(char *);
void assemble_rr(char *);
void assemble_sla(char *);
void assemble_sra(char *);
void assemble_sli(char *);
void assemble_srl(char *);
void assemble_rrd(char *);
void assemble_rld(char *);

void assemble_ldi(char *);
void assemble_ldd(char *);
void assemble_ldir(char *);
void assemble_lddr(char *);
void assemble_cpir(char *);
void assemble_cpdr(char *);

void assemble_nop(char *);
void assemble_halt(char *);
void assemble_di(char *);
void assemble_ei(char *);

void assemble_im(char *);
void assemble_jp(char *);
void assemble_jr(char *);

void assemble_call(char *);
void assemble_djnz(char *);

void assemble_rst(char *);
void assemble_ret(char *);
void assemble_reti(char *);
void assemble_retn(char *);

void assemble_bit(char *);
void assemble_set(char *);
void assemble_res(char *);

void assemble_in(char *);
void assemble_out(char *);

void assemble_ini(char *);
void assemble_ind(char *);
void assemble_outi(char *);
void assemble_outd(char *);
void assemble_inir(char *);
void assemble_indr(char *);
void assemble_otir(char *);
void assemble_otdr(char *);



#define AT_8BITREG 0
#define AT_NUMCONST 1
#define AT_16BITREG 2
#define AT_16BITCONST 3
#define AT_MEMPTR 4

#define AT_MASK 0xf

#define AT_SYMBOLIC 128

struct arg
{
    unsigned char type;
    unsigned char prefixed;
    unsigned char relocate;
    unsigned int offset;
    long value;
    char *str;
};

void parsearg(char *name, struct arg * arg);
