

;		HL - pointer to console base

s_ccon_alloc
    push hl			; store console base
    ld de, #256			; one page, 32*8
    call (s_alloc)		; allocate memory for characters

    push hl
    pop bc			; copy memory pointer to BC
    
    ld de, #256
    ld a, #32			; space
    call (s_util_fillblock)	; clear console data
    
    ex de, hl			; move console data pointer to DE
    pop hl			; and restore console base
    
    ld (hl), e
    inc hl
    ld (hl), d			; save console data pointer
    inc hl
    
    sub a
    ld (hl), a			; set write offset to 0
    inc hl
    ld (hl), a			; input buffer size = 0
    inc hl
    ld (hl), a			; input buffer write offset = 0
    inc hl
    ld (hl), a			; input buffer read offset = 0
    
    ret				; done

s_ccon_refresh
    ld hl, (#24154)		; current console
    ld e, (hl)
    inc hl
    ld d, (hl)			; load console data pointer into DE
    ex de, hl			; and save it into HL
    
    ld b, #0			; row
cr_loop1

    ld c, #0			; column
cr_loop2
    ld a, (hl)			; load character
    push bc			; save coordinates
    call (s_console_paste)	; paste character
    pop bc			; restore coordinates

    inc hl			; proceed to next character
    inc c			; next column
    ld a, c
    and 0x20			; 32
    jr z, cr_loop2		; continue if less than 32
    inc b			; next row
    ld a, b
    and 0x08			; 8
    jr z, cr_loop1		; continue if less than 8
    ret				; all done

;		A - console number
s_ccon_syschvt
    push de
    pop af
s_ccon_chvt
    and 0x03
    ld l, a
    ld h, #0
    add hl, hl			; *2
    add hl, hl			; *4
    add hl, hl			; *8
    add hl, hl			; *16
    ld de,#24090		; first console
    add hl, de			; calculate address of console base
    ld (#24154), hl		; save calculated address as current console addr
    jr s_ccon_refresh		; update screen

s_ccon_init
    ld hl, #24090
    ld (#24154), hl
    call (s_ccon_alloc)
    ld hl, #24106
    call (s_ccon_alloc)
    ld hl, #24122
    call (s_ccon_alloc)
    ld hl, #24138
    call (s_ccon_alloc)
    
    ret
#
# s_ccon_write  -  add a character to console
#

s_ccon_common1
    ld hl, (#24004)		; get current task structure

    inc hl
    inc hl			; skip next task pointer
    inc hl
    inc hl			; skip stack frame pointer
    ld e, (hl)			; read console pointer
    inc hl
    ld d, (hl)			; now console pointer is at de
    ex de, hl			; now it's in hl


    push hl
    pop bc			; copy it to bc

    inc hl			; skip LSB of console data pointer
    ld d, (hl)			; read MSB of console data pointer
    inc hl
    ret

s_ccon_syswrite
    push de
    pop af
s_ccon_write
    call (s_ccon_common1)
    ld e, (hl)			; read current offset and take it as LSB of cdp
    
    cp 10			; \n ?
    jr nz, cw_notlf
    ld a, e
    add a, #32			; next line
    jr c, cw_scroll		; too much

    and 0xe0

    ld (hl), a
    ret

cw_notlf
    ld (de), a			; paste the character
    
    inc e
    jr z, cw_scroll		; we need scrolling
    
    ld (hl), e

s_ccon_common2

    scf
    ccf				; zero carry flag

    di				; critical operations, stop tasking
    ld hl, (#24154)		; current console
    sbc hl, bc
    jr nz, cw_notcurrent	; is our console the current one ?
    
    push af			; save character
    dec e			; fall back to previous char

    ld a, e
    and 0x1f			; calculate column #
    ld c, a
    ld a, e
    
    rlca
    rlca
    rlca			; upper three bits
    and 0x07			; calculate line #
    ld b, a
    pop af			; restore character code
    call (s_console_paste)	; write character
cw_notcurrent
    ei				; restore tasking
    ret
cw_scroll
    ld (hl), #224		; line 7 col 0
    push hl			; store cptr address
    ld h, d			; copy DE -> HL

    ld l, #32
    ld e, #0			; initialize: copy 1st line to line #0
    ld bc, #224			; 7 lines
    ldir			; copy block
    
    push de			; DE holds now the address of eight line
    pop bc			; copy it to BC
    ld a, #32			; space
    ld de, #32			; 32 bytes len
    call (s_util_fillblock)	; clear line

    pop bc			; restore cptr address
    
    dec bc
    dec bc			; restore valid cptr pointer

    scf
    ccf				; zero carry flag

    ld hl, (#24154)		; current console
    sbc hl, bc
    ret nz			; if our console is not the current one
    jp (s_ccon_refresh)

#
# s_ccon_waitkey  -  read a character from console buffer
#

s_ccon_waitkey
    ld hl, (#24004)		; get current task structure

    inc hl
    inc hl			; skip next task pointer
    inc hl
    inc hl			; skip stack frame pointer
    ld e, (hl)			; read console pointer
    inc hl
    ld d, (hl)			; now console pointer is at de

    ex de, hl			; now it's at hl

    inc hl
    inc hl
    inc hl			; move to input buffer data
    
    sub a
    
wk_waitloop
    cp (hl)			; any data there ?
    jr nz, wk_gotkey
    di
    call (s_interrupt)		; give away control
    jr wk_waitloop

wk_gotkey
    dec (hl)			; decrement character count
    inc hl			; now we are at write offset
    inc hl			; now we are at read offset

    ld a, (hl)			; read read offset
    inc (hl)			; increment read offset
    and 0x07			; make sure it has a reasonable value
    
    inc hl			; increment
    
    add a, l			; we can skip H
    ld l, a
    
    ld l, (hl)			; read the actual character

    ret

#
# s_ccon_passchar  -  add a character to console buffer
#


s_ccon_passchar
    push af
    
    ld hl, (#24154)		; current console
    inc hl
    inc hl
    inc hl			; skip data & offset pointers
    
    ld a, (hl)
    cp 7
    jr nz, pc_lenok

    pop af			; too many characters
    ret
pc_lenok
    inc (hl)			; input buffer size
    
    inc hl			; move to input buffer write offset
    
    ld a, (hl)			; load offset
    inc (hl)			; increment offset
    and 0x07			; make sure offset is valid
    
    inc hl			; skip buffer read offset
    inc hl			; increment
    
    add a, l			; we can skip H
    ld l, a
    
    pop af
    ld (hl), a			; load the character into buffer
    ret				; done

s_ccon_bkspc
    call (s_ccon_common1)
    dec (hl)
    ld e, (hl)			; read current offset and take it as LSB of cdp
    ld a, #32
    ld (de), a
    inc e
    jp (s_ccon_common2)

!global s_ccon_init
!global s_ccon_refresh
!global s_ccon_chvt
!global s_ccon_write
!global s_ccon_waitkey
!global s_ccon_passchar
!global s_ccon_syswrite
!global s_ccon_syschvt
!global s_ccon_bkspc
